package com.moovapps.process.extensions;

import com.axemble.easysite.dao.domain.impl.BlockImpl;
import com.axemble.sdk.blocks.sys.layouts.SimpleBlock;
import com.axemble.sdk.components.sys.navigation.HyperlinkComponent;
import com.axemble.sdk.components.sys.texts.HeadingComponent;
import com.axemble.sdk.components.sys.views.DataComponent;
import com.axemble.vdoc.sdk.Modules;
import com.axemble.vdoc.sdk.exceptions.SiteModuleException;
import com.axemble.vdoc.sdk.interfaces.IBlock;
import com.axemble.vdoc.sdk.interfaces.IContent;
import com.axemble.vdoc.sdk.interfaces.IContext;
import com.axemble.vdoc.sdk.interfaces.IPageContainer;
import com.axemble.vdoc.sdk.modules.ISiteModule;
import com.moovapps.workplace.sdk.beans.IApplication;
import com.moovapps.workplace.sdk.beans.types.ApplicationType;
import com.moovapps.workplace.sdk.extension.base.BaseApplicationBuilderExtension;
import org.apache.commons.lang3.Validate;

import java.util.ArrayList;
import java.util.List;

public class AddressViewBuilderExtension extends BaseApplicationBuilderExtension {

    private static final com.axemble.vdoc.sdk.utils.Logger LOG = com.axemble.vdoc.sdk.utils.Logger.getLogger(AddressViewBuilderExtension.class);

    @Override
    public void onAfterUpdateContent(IApplication application)
    {
        //Getting a site module
        ISiteModule siteModule = Modules.getSiteModule();
        try {

            //We allow treatment only if the application is a process-type application
            Validate.isTrue(ApplicationType.PROCESS.getKey().equals(application.getType().getKey()));

            //Treatment will be done in a transaction
            siteModule.beginTransaction();

            //Treatment will be done as sysadmin
            IContext context = siteModule.getSysadminContext();

            //Getting the draft content for the page
            IPageContainer applicationPage = getPageContainer(application);
            IContent draftContent = applicationPage.getContent(IContent.IStatus.DRAFT);

            //Add the storage view to the page
            addStorageView(siteModule, draftContent);
            draftContent.save(context);

            //Approve the page
            siteModule.approve(context, applicationPage);

            siteModule.commitTransaction();

        } catch (SiteModuleException e) {
            LOG.error(e);
        } finally {
            if (siteModule.isTransactionActive()) {
                siteModule.rollbackTransaction();
            }
            Modules.releaseModule(siteModule);
        }
    }

    private void addStorageView(ISiteModule siteModule, IContent draftContent) throws SiteModuleException {
        //Getting the custom part of the menu and clearing it
        IBlock menuBlock = draftContent.getBlock().findBlockBySystemName("menu-custom", true);
        ((BlockImpl) menuBlock.getNativeObject()).getAdditionalNodeList().clear();

        //Creating a block and a link inside it
        SimpleBlock simpleBlock = siteModule.getBlocksFactory().newSimpleBlock();
        simpleBlock.setInnerName("MENU-STORAGE_VIEW");
        simpleBlock.setInnerLabel("STORAGE_VIEW");
        HyperlinkComponent hyperlink = siteModule.getComponentsFactory().newHyperlinkComponent();
        hyperlink.setLabel("Mes adresses");
        hyperlink.setTarget("_self");
        //the block parameter in the menu link will be used by the page to display the matching content <-- Important
        hyperlink.setValue("href","?block=STORAGE_VIEW");
        simpleBlock.addComponent(hyperlink);
        menuBlock.addBlock(simpleBlock);

        //Getting the custom part of the content and clearing it
        IBlock contentBlock = draftContent.getBlock().findBlockBySystemName("content-custom", true);
        ((BlockImpl) contentBlock.getNativeObject()).getAdditionalNodeList().clear();

        //Creating the content
        //We need a block with system name matching the parameter of the link from the menu <-- Important
        SimpleBlock contentSimpleBlock = siteModule.getBlocksFactory().newSimpleBlock();
        contentSimpleBlock.setInnerName("CONTENT-STORAGE_VIEW");
        contentSimpleBlock.setInnerLabel("STORAGE_VIEW");
        //We put a header on top of the view
        HeadingComponent headingComponent = siteModule.getComponentsFactory().newHeadingComponent();
        headingComponent.setLevel("1");
        headingComponent.setContent("Mes adresses");
        contentSimpleBlock.addComponent(headingComponent);
        //We add a component to show a view and parameterize it
        DataComponent dataComponent = siteModule.getComponentsFactory().newDataComponent();
        dataComponent.setDefaultDisplayMode("list");
        dataComponent.setNbElementsPerPage("25");
        dataComponent.setUsePlugin(false);
        List<String> viewList = new ArrayList();
        viewList.add("uril://vdoc/resourceDefinitionView/DefaultOrganization/MoovappsWorkplace/Companies:4/Address/DEFAULT");
        dataComponent.setViewList(viewList);
        contentSimpleBlock.addComponent(dataComponent);

        //We add our content to the custom content block
        contentBlock.addBlock(contentSimpleBlock);
    }
}
