package com.moovapps.process.extensions;

import com.axemble.vdoc.sdk.Modules;
import com.axemble.vdoc.sdk.exceptions.SiteModuleException;
import com.axemble.vdoc.sdk.interfaces.IContext;
import com.axemble.vdoc.sdk.interfaces.IPageContainer;
import com.axemble.vdoc.sdk.interfaces.ISite;
import com.axemble.vdoc.sdk.interfaces.ITopic;
import com.axemble.vdoc.sdk.modules.ISiteModule;
import com.moovapps.workplace.sdk.beans.IApplication;
import com.moovapps.workplace.sdk.beans.types.ApplicationType;
import com.moovapps.workplace.sdk.extension.base.BaseApplicationBuilderExtension;
import com.moovapps.workplace.sdk.modules.IWorkplaceModule;
import com.moovapps.workplace.sdk.modules.factory.WorkplaceModuleFactory;
import org.apache.commons.lang3.Validate;

public class ThematicBuilderExtension extends BaseApplicationBuilderExtension {

    private static final com.axemble.vdoc.sdk.utils.Logger LOG = com.axemble.vdoc.sdk.utils.Logger.getLogger(ThematicBuilderExtension.class);

    @Override
    public void onAfterUpdateContent(IApplication application)
    {
        //Getting a site module
        ISiteModule siteModule = Modules.getSiteModule();
        try {
            //We are going to use the parameters
            Validate.notNull(application.getParameters());

            //We allow treatment only if the application is a link-type application
            Validate.isTrue(ApplicationType.LINK.getKey().equals(application.getType().getKey()));

            //Treatment will be done in a transaction
            siteModule.beginTransaction();

            //We do the treatment as sysadmin
            IContext context = siteModule.getSysadminContext();

            //Retreieving workplace site
            IWorkplaceModule workplaceModule = WorkplaceModuleFactory.getWorkplaceModule();
            ISite site = siteModule.getSiteByName(context, workplaceModule.getWorkplaceSiteName());

            //We check if the content we want to make already exists and delete it
            ITopic topic = siteModule.getTopic(context,getCustomTopic(),application.getParameters());
            if(topic != null) {
                IPageContainer container = siteModule.getPageContainer(context,topic,"index");
                if(container != null){
                    siteModule.delete(context,container);
                }
                siteModule.delete(context,topic);
            }

            //We create some content (by copying a custom page)
            topic = siteModule.createTopic(context, getCustomTopic(), application.getParameters(), application.getParameters());
            IPageContainer templatePage = siteModule.getPageContainer(context, site, "custom/thematic/index");
            siteModule.copyPageContainerInTopic(context, templatePage, topic);

            //We approve the new content
            IPageContainer container = siteModule.getPageContainer(context,topic,"index");
            siteModule.approve(context, container);

            //We update the application to have its link pointed at
            application.setLink(container.getProtocolURI());

            siteModule.commitTransaction();

        } catch (SiteModuleException e) {
            LOG.error(e);
        } finally {
            if (siteModule.isTransactionActive()) {
                siteModule.rollbackTransaction();
            }
            Modules.releaseModule(siteModule);
        }
    }
}
